@@ -330,6 +330,13 @@ class Mechanize::Form
     @clicked_buttons << button
   end
 
+  # This method allows the same form to be submitted second time
+  # with the different submit button being clicked.
+  def reset
+    # In the future, should add more functionality here to reset the form values to their defaults.
+    @clicked_buttons = []
+  end
+
   # This method calculates the request data to be sent back to the server
   # for this form, depending on if this is a regular post, get, or a
   # multi-part post,
@@ -348,8 +355,7 @@ class Mechanize::Form
       params.concat @file_uploads.map { |f| file_to_multipart(f) }
 
       params.map do |part|
-        part.force_encoding('ASCII-8BIT') if part.respond_to? :force_encoding
-        "--#{boundary}\r\n#{part}"
+        "--#{boundary}\r\n#{part.force_encoding(Encoding::ASCII_8BIT)}"
       end.join('') +
         "--#{boundary}--\r\n"
     else
