@@ -167,7 +167,7 @@ class Mechanize::HTTP::Agent
 
     @scheme_handlers = Hash.new { |h, scheme|
       h[scheme] = lambda { |link, page|
-        raise Mechanize::UnsupportedSchemeError, scheme
+        raise Mechanize::UnsupportedSchemeError.new(scheme, link)
       }
     }
 
@@ -213,6 +213,10 @@ class Mechanize::HTTP::Agent
   # +method+ is used to retrieve it, along with the HTTP +headers+, request
   # +params+ and HTTP +referer+.
   #
+  # The final URI to access is built with +uri+ and +params+, the
+  # latter of which is formatted into a string using
+  # Mechanize::Util.build_query_string, which see.
+  #
   # +redirects+ tracks the number of redirects experienced when retrieving the
   # page.  If it is over the redirection_limit an error will be raised.
 
@@ -246,9 +250,13 @@ class Mechanize::HTTP::Agent
       request['If-Modified-Since'] = last_modified
     end if @conditional_requests
 
-    # Specify timeouts if given
-    connection.open_timeout = @open_timeout if @open_timeout
-    connection.read_timeout = @read_timeout if @read_timeout
+    # Specify timeouts if supplied and our connection supports them
+    if @open_timeout && connection.respond_to?(:open_timeout=)
+      connection.open_timeout = @open_timeout
+    end
+    if @read_timeout && connection.respond_to?(:read_timeout=)
+      connection.read_timeout = @read_timeout
+    end
 
     request_log request
 
@@ -515,6 +523,8 @@ class Mechanize::HTTP::Agent
 
   def request_auth request, uri
     base_uri = uri + '/'
+    base_uri.user     = nil
+    base_uri.password = nil
     schemes = @authenticate_methods[base_uri]
 
     if realm = schemes[:digest].find { |r| r.uri == base_uri } then
@@ -530,9 +540,7 @@ class Mechanize::HTTP::Agent
   def request_auth_digest request, uri, realm, base_uri, iis
     challenge = @digest_challenges[realm]
 
-    user, password, = @auth_store.credentials_for uri, realm.realm
-    uri.user     = user
-    uri.password = password
+    uri.user, uri.password, = @auth_store.credentials_for uri, realm.realm
 
     auth = @digest_auth.auth_header uri, challenge.to_s, request.method, iis
     request['Authorization'] = auth
@@ -608,17 +616,7 @@ class Mechanize::HTTP::Agent
       end
 
       url.gsub!(/[^#{0.chr}-#{126.chr}]/o) { |match|
-        if RUBY_VERSION >= "1.9.0"
-          Mechanize::Util.uri_escape(match)
-        else
-          begin
-            sprintf('%%%X', match.unpack($KCODE == 'UTF8' ? 'U' : 'C').first)
-          rescue ArgumentError
-            # workaround for ruby 1.8 with -Ku but ISO-8859-1 characters in
-            # URIs.  See #227.  I can't wait to drop 1.8 support
-            sprintf('%%%X', match.unpack('C').first)
-          end
-        end
+        Mechanize::Util.uri_escape(match)
       }
 
       escaped_url = Mechanize::Util.html_unescape(
@@ -679,6 +677,18 @@ class Mechanize::HTTP::Agent
     uri
   end
 
+  def secure_resolve!(uri, referer = current_page)
+    new_uri = resolve(uri, referer)
+
+    if (referer_uri = referer && referer.uri) &&
+       referer_uri.scheme != 'file'.freeze &&
+       new_uri.scheme == 'file'.freeze
+      raise Mechanize::Error, "insecure redirect to a file URI"
+    end
+
+    new_uri
+  end
+
   def resolve_parameters uri, method, parameters
     case method
     when :head, :get, :delete, :trace then
@@ -740,7 +750,7 @@ class Mechanize::HTTP::Agent
 
       if existing_realms.include? realm
         message = 'Digest authentication failed'
-        raise Mechanize::UnauthorizedError.new(page, challeges, message)
+        raise Mechanize::UnauthorizedError.new(page, challenges, message)
       end
 
       existing_realms << realm
@@ -822,7 +832,7 @@ class Mechanize::HTTP::Agent
   ensure
     begin
       if Tempfile === body_io and
-         (StringIO === out_io or out_io.path != body_io.path) then
+         (StringIO === out_io or (out_io and out_io.path != body_io.path)) then
         body_io.close!
       end
     rescue IOError
@@ -861,7 +871,7 @@ class Mechanize::HTTP::Agent
   def response_follow_meta_refresh response, uri, page, redirects
     delay, new_url = get_meta_refresh(response, uri, page)
     return nil unless delay
-    new_url = new_url ? resolve(new_url, page) : uri
+    new_url = new_url ? secure_resolve!(new_url, page) : uri
 
     raise Mechanize::RedirectLimitReachedError.new(page, redirects) if
       redirects + 1 > @redirection_limit
@@ -893,10 +903,9 @@ class Mechanize::HTTP::Agent
     if use_tempfile? content_length then
       body_io = make_tempfile 'mechanize-raw'
     else
-      body_io = StringIO.new
+      body_io = StringIO.new.set_encoding(Encoding::BINARY)
     end
 
-    body_io.set_encoding Encoding::BINARY if body_io.respond_to? :set_encoding
     total = 0
 
     begin
@@ -970,8 +979,9 @@ class Mechanize::HTTP::Agent
       headers.delete key
     end
 
+    new_uri = secure_resolve! response['Location'].to_s, page
+
     @history.push(page, page.uri)
-    new_uri = resolve response['Location'].to_s, page
 
     fetch new_uri, redirect_method, headers, [], referer, redirects + 1
   end
@@ -1086,12 +1096,12 @@ class Mechanize::HTTP::Agent
   # SSL version to use
   def ssl_version
     @http.ssl_version
-  end if RUBY_VERSION > '1.9'
+  end
 
   # Sets the SSL version to use
   def ssl_version= ssl_version
     @http.ssl_version = ssl_version
-  end if RUBY_VERSION > '1.9'
+  end
 
   # A callback for additional certificate verification.  See
   # OpenSSL::SSL::SSLContext#verify_callback
@@ -1141,9 +1151,7 @@ class Mechanize::HTTP::Agent
   # processing.
 
   def auto_io name, read_size, input_io
-    out_io = StringIO.new
-
-    out_io.set_encoding Encoding::BINARY if out_io.respond_to? :set_encoding
+    out_io = StringIO.new.set_encoding(Encoding::BINARY)
 
     until input_io.eof? do
       if StringIO === out_io and use_tempfile? out_io.size then
@@ -1216,7 +1224,7 @@ class Mechanize::HTTP::Agent
   def make_tempfile name
     io = Tempfile.new name
     io.unlink
-    io.binmode if io.respond_to? :binmode
+    io.binmode
     io
   end
 
