@@ -13,12 +13,6 @@ class TestMechanize < Mechanize::TestCase
     @res = Net::HTTPOK.allocate
     @res.instance_variable_set :@code, 200
     @res.instance_variable_set :@header, {}
-
-    @headers = if RUBY_VERSION > '1.9' then
-                 %w[accept user-agent]
-               else
-                 %w[accept]
-               end
   end
 
   def test_back
@@ -112,6 +106,22 @@ class TestMechanize < Mechanize::TestCase
     assert_equal '/index.html', requests.first.path
   end
 
+  def test_click_image_button
+    page = @mech.get("http://localhost/form_test.html")
+    get_form = page.forms.find { |f| f.name == "get_form1" }
+    image_button = get_form.buttons.first
+    new_page = @mech.click(image_button)
+    assert_equal "http://localhost/form_post?first_name=&button.x=0&button.y=0", new_page.uri.to_s
+  end
+
+  def test_click_submit_button
+    page = @mech.get("http://localhost/form_test.html")
+    get_form = page.forms.find { |f| f.name == "get_form1" }
+    submit_button = get_form.submits.first
+    new_page = @mech.click(submit_button)
+    assert_equal "http://localhost/form_post?first_name=", new_page.uri.to_s
+  end
+
   def test_click_frame
     frame = node 'frame', 'src' => '/index.html'
     frame = Mechanize::Page::Frame.new frame, @mech, fake_page
@@ -433,10 +443,6 @@ but not <a href="/" rel="me nofollow">this</a>!
 
   def test_get_follow_meta_refresh_anywhere
     @mech.follow_meta_refresh = :anywhere
-    requests = []
-    @mech.pre_connect_hooks << lambda { |_, request|
-      requests << request
-    }
 
     @mech.get('http://localhost/tc_meta_in_body.html')
     assert_equal 2, requests.length
@@ -465,10 +471,6 @@ but not <a href="/" rel="me nofollow">this</a>!
 
   def test_get_follow_meta_refresh_in_body
     @mech.follow_meta_refresh = true
-    requests = []
-    @mech.pre_connect_hooks << lambda { |_, request|
-      requests << request
-    }
 
     @mech.get('http://localhost/tc_meta_in_body.html')
     assert_equal 1, requests.length
@@ -492,12 +494,6 @@ but not <a href="/" rel="me nofollow">this</a>!
   def test_get_follow_meta_refresh_referer_not_sent
     @mech.follow_meta_refresh = true
 
-    requests = []
-
-    @mech.pre_connect_hooks << lambda { |_, request|
-      requests << request
-    }
-
     @mech.get('http://localhost/tc_follow_meta.html')
 
     assert_equal 2, @mech.history.length
@@ -578,12 +574,6 @@ but not <a href="/" rel="me nofollow">this</a>!
   def test_get_http_refresh
     @mech.follow_meta_refresh = true
 
-    requests = []
-
-    @mech.pre_connect_hooks << lambda { |_, request|
-      requests << request
-    }
-
     page = @mech.get('http://example/http_refresh?refresh_time=0')
 
     assert_equal('http://example/', page.uri.to_s)
@@ -609,18 +599,15 @@ but not <a href="/" rel="me nofollow">this</a>!
     assert_equal('http://localhost/http_refresh?refresh_time=0', page.uri.to_s)
   end
 
-  def test_get_kcode
-    $KCODE = 'u'
-    page = @mech.get("http://localhost/?a=#{[0xd6].pack('U')}")
-
-    assert_equal('http://localhost/?a=%D6', page.uri.to_s)
-
-    $KCODE = 'NONE'
-  end unless RUBY_VERSION >= '1.9.0'
-
   def test_get_query
     page = @mech.get('http://localhost/', { :q => 'hello' })
     assert_equal('http://localhost/?q=hello', page.uri.to_s)
+
+    page = @mech.get('http://localhost/', { :q => %w[hello world]})
+    assert_equal('http://localhost/?q=hello&q=world', page.uri.to_s)
+
+    page = @mech.get('http://localhost/', { :paging => { start: 1, limit: 25 } })
+    assert_equal('http://localhost/?paging%5Bstart%5D=1&paging%5Blimit%5D=25', page.uri.to_s)
   end
 
   def test_get_redirect
@@ -665,11 +652,6 @@ but not <a href="/" rel="me nofollow">this</a>!
   end
 
   def test_get_referer_none
-    requests = []
-    @mech.pre_connect_hooks << lambda { |_, request|
-      requests << request
-    }
-
     @mech.get('http://localhost/')
     @mech.get('http://localhost/')
     assert_equal(2, requests.length)
@@ -680,7 +662,13 @@ but not <a href="/" rel="me nofollow">this</a>!
 
   def test_get_scheme_unsupported
     assert_raises Mechanize::UnsupportedSchemeError do
-      @mech.get('ftp://server.com/foo.html')
+      begin
+        @mech.get('ftp://server.com/foo.html')
+      rescue Mechanize::UnsupportedSchemeError => error
+        assert_equal 'ftp', error.scheme
+        assert_equal 'ftp://server.com/foo.html', error.uri.to_s
+        raise
+      end
     end
   end
 
@@ -942,6 +930,25 @@ but not <a href="/" rel="me nofollow">this</a>!
     assert page.body.length > File.read(__FILE__).length
   end
 
+  def test_post_file_upload
+    name = File.basename(__FILE__)
+    file_upload = Mechanize::Form::FileUpload.new({'name' => 'userfile1'}, name)
+    file_upload.file_data = File.read(__FILE__)
+    file_upload.mime_type = 'application/zip'
+
+    page = @mech.post('http://localhost/file_upload', {
+      :name       => 'Some file',
+      :userfile1  => file_upload
+    })
+
+    assert_match(
+      "Content-Disposition: form-data; name=\"userfile1\"; filename=\"#{name}\"",
+      page.body
+    )
+    assert_match("Content-Type: application/zip", page.body)
+    assert page.body.length > File.read(__FILE__).length
+  end
+
   def test_post_redirect
     page = @mech.post('http://localhost/redirect')
 
@@ -970,6 +977,13 @@ but not <a href="/" rel="me nofollow">this</a>!
     assert_equal 5, @mech.read_timeout
   end
 
+  def test_timeouts_for_file_connection
+    uri = URI.parse "file://#{File.expand_path __FILE__}"
+    @mech.read_timeout = 5
+    @mech.open_timeout = 5
+    assert @mech.get(uri)
+  end
+
   def test_referer
     host_path = "localhost/tc_referer.html?t=1"
     ['http', 'https'].each { |proto|
