@@ -18,10 +18,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     @headers = if RUBY_VERSION >= '2.0.0' then
                  %w[accept accept-encoding user-agent]
-               elsif RUBY_VERSION >= '1.9.0' then
-                 %w[accept user-agent]
                else
-                 %w[accept]
+                 %w[accept user-agent]
                end
   end
 
@@ -33,6 +31,16 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     realm
   end
 
+  def jruby_zlib?
+    if RUBY_ENGINE == 'jruby'
+      meth = caller[0][/`(\w+)/, 1]
+      warn "#{meth}: skipped because how Zlib handles error is different in JRuby"
+      true
+    else
+      false
+    end
+  end
+
   def test_agent_is_named
     assert_equal 'mechanize', Mechanize::HTTP::Agent.new.http.name
     assert_equal 'unique', Mechanize::HTTP::Agent.new('unique').http.name
@@ -494,6 +502,12 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     assert_match %r%^Digest %, @req['Authorization']
     assert_match %r%qop=auth%, @req['Authorization']
+
+    @req['Authorization'] = nil
+    @agent.request_auth @req, @uri
+
+    assert_match %r%^Digest %, @req['Authorization']
+    assert_match %r%qop=auth%, @req['Authorization']
   end
 
   def test_request_auth_iis_digest
@@ -913,6 +927,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     body_io = StringIO.new \
       "\037\213\b\0002\002\225M\000\003+H,*\001"
 
+    return if jruby_zlib?
+
     e = assert_raises Mechanize::Error do
       @agent.response_content_encoding @res, body_io
     end
@@ -944,6 +960,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert body_io.closed?
 
     assert_match %r%invalid compressed data -- crc error%, log.string
+  rescue IOError
+    raise unless jruby_zlib?
   end
 
   def test_response_content_encoding_gzip_checksum_corrupt_length
@@ -960,6 +978,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert body_io.closed?
 
     assert_match %r%invalid compressed data -- length error%, log.string
+  rescue IOError
+    raise unless jruby_zlib?
   end
 
   def test_response_content_encoding_gzip_checksum_truncated
@@ -976,6 +996,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert body_io.closed?
 
     assert_match %r%unable to gunzip response: footer is not found%, log.string
+  rescue IOError
+    raise unless jruby_zlib?
   end
 
   def test_response_content_encoding_gzip_empty
@@ -1015,6 +1037,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal 'part', body.read
 
     assert body_io.closed?
+  rescue IOError
+    raise unless jruby_zlib?
   end
 
   def test_response_content_encoding_none
@@ -1193,6 +1217,24 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     end
   end
 
+  def test_response_meta_refresh_with_insecure_url
+    uri = URI.parse 'http://example/#id+1'
+
+    body = <<-BODY
+<title></title>
+<meta http-equiv="refresh" content="0; url=file:///dev/zero">
+    BODY
+
+    page = Mechanize::Page.new(uri, nil, body, 200, @mech)
+
+    @agent.follow_meta_refresh = true
+
+    assert_raises Mechanize::Error do
+      @agent.response_follow_meta_refresh(@res, uri, page,
+                                          @agent.redirection_limit)
+    end
+  end
+
   def test_response_parse
     body = '<title>hi</title>'
     @res.instance_variable_set :@header, 'content-type' => %w[text/html]
@@ -1288,7 +1330,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     body = io.read
 
     assert_equal 'part', body
-    assert_equal Encoding::BINARY, body.encoding if body.respond_to? :encoding
+    assert_equal Encoding::BINARY, body.encoding
   end
 
   def test_response_read_chunked_no_trailer
@@ -1368,17 +1410,15 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
       io = @agent.response_read res, req, uri
 
-      expected = "π\n"
-      expected.force_encoding Encoding::BINARY if expected.respond_to? :encoding
+      expected = "π\n".force_encoding(Encoding::BINARY)
 
       # Ruby 1.8.7 doesn't let us set the write mode of the tempfile to binary,
       # so we should expect an inserted carriage return on some platforms
-      expected_with_carriage_return = "π\r\n"
-      expected_with_carriage_return.force_encoding Encoding::BINARY if expected_with_carriage_return.respond_to? :encoding
+      expected_with_carriage_return = "π\r\n".force_encoding(Encoding::BINARY)
 
       body = io.read
       assert_match(/^(#{expected}|#{expected_with_carriage_return})$/m, body)
-      assert_equal Encoding::BINARY, body.encoding if body.respond_to? :encoding
+      assert_equal Encoding::BINARY, body.encoding
     end
   end
 
@@ -1479,6 +1519,16 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal 'http://example/referer', requests.first['Referer']
   end
 
+  def test_response_redirect_insecure
+    @agent.redirect_ok = true
+    referer = page 'http://example/referer'
+
+    assert_raises Mechanize::Error do
+      @agent.response_redirect({ 'Location' => 'file:///etc/passwd' }, :get,
+                               fake_page, 0, {}, referer)
+    end
+  end
+
   def test_response_redirect_limit
     @agent.redirect_ok = true
     referer = page 'http://example/referer'
@@ -1601,7 +1651,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       @agent.cert_store = store
       @agent.certificate = ssl_certificate
       @agent.private_key = ssl_private_key
-      @agent.ssl_version = 'SSLv3' if RUBY_VERSION > '1.9'
+      @agent.ssl_version = 'SSLv3'
       @agent.verify_callback = proc { |ok, context| }
 
       http = @agent.http
@@ -1610,8 +1660,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       assert_equal store,                     http.cert_store
       assert_equal ssl_certificate,           http.certificate
       assert_equal ssl_private_key,           http.private_key
-      assert_equal 'SSLv3',                   http.ssl_version if
-        RUBY_VERSION > '1.9'
+      assert_equal 'SSLv3',                   http.ssl_version
       assert_equal OpenSSL::SSL::VERIFY_PEER, http.verify_mode
       assert http.verify_callback
     end
